# The dataset

The dataset represents a partial dump of the FINALES database. It consists of three files, whose content is overwritten from one version to the next:
- capabilities.json
- requests.json
- results_for_requests.json.
- sqlite.db
- db_schema.png.
- simplified_results_singletask_ontologized.jsonld

capabilities.json corresponds to the FINALES' API end point (GET, /capabilities), 
which returns all (currently available) capabilities registered in the MAP. 

requests.json corresponds to the end point (GET, /all_requests), 
which returns all requests that were posted by the tenants. 

results_for_requests.json corresponds to the end point (GET, /results_requested), 
which returns all collected results for the posted requests.

sqlite.db is a copy of the database file, which was retrieved via the end point (GET, /database_dump/<access_key>). See sections below for more information. 

db_schema.png shows a schema for the FINALES database.

simplified_results_singletask_ontologized.jsonld presents the results in ontologised
representation. Consult the section 'Ontologized results - simplified_results_singletask_ontologized.jsonld' further down the README.


# Electrode materials

The materials of the electrodes are recorded in the database as pure graphite and pure lithium nickel dioxide (LNO) for the anode and cathode, respectively. However, these materials typically contain binders and other materials technically required for their use in battery cells. Hence, the records in the database regarding electrode materials should be understood as denoting the type of the electrode rather than its actual composition on a molecular level.

# Extracting the dataset from the FINALES database

The dataset is produced by executing the command `bma finales-db back-up`. 
Detailed [instructions](https://github.com/materialscloud-org/big-map-archive-api-client/blob/master/README.md#installation) on how to install the corresponding CLI app locally 
and a list of the available [options](https://github.com/materialscloud-org/big-map-archive-api-client/blob/master/README.md#back-up-finales-databases) for that command 
can be found on GitHub. 
The used [input file](https://github.com/materialscloud-org/big-map-archive-api-client-finales/blob/master/final_run_2023/metadata.yaml) named metadata.yaml, which contains the title, the authors and their affiliations, etc., 
can be found in a different repository on the same platform.

The command is executed periodically, typically once per day during the study. 
Each new execution creates and publishes a new version of the entry in the BIG-MAP Archive data repository. 
To determine the date when the command was run for a specific version, check the version's publication date.

# The sqlite.db file

The binary database file sqlite.db is a dump of the entire database for the FINALES instance pulled directly from an API endpoint. The tables that make up the database are the following:
- quantity: this table contains all active and non-active capabilities that have been registered with the MAP
- tenant: this table stores all tenants that have been registered and coupled to one or several quantities - all the limitations within each capability is stated for each tenant
- request: all posted requests
- result: all posted results
- link_quantity_request: uuid links between quantities and requests
- link_quantity_result: uuid links between quantities and results
- status_log_request: logs when the status of a request is changed
- status_log_result: logs when the status of a result is changed.

For viewing the database schema, including foreign keys, open the db_schema.png file. If you wish to easily browse through the contents of the database, you can use an online tool such as https://sqliteviewer.app.

# Examples of database queries

The SQLite database offers a fairly simple functionalities for retrieving data. We provide two examples of queries. In the first example, we show how to retrieve data from the result table.

````
# Necessary package
import sqlite3

# Define file name
db_file = "sqlite.db"

# Connect to database
conn = sqlite3.connect(db_file)

# Create cursors
cursor = conn.cursor()

# Fetch the list of tables from database
cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
tables = cursor.fetchall()

# Fetch all rows from the table "result"
cursor.execute(f"SELECT * FROM result;")
rows = cursor.fetchall()

for i, row in enumerate(rows):
    print(f"row number {i}\n{row}")

# Close the connections
conn.close()
````

In the second example, we add information from the tenant table.

````
# Necessary package
import sqlite3

# Define file name
db_file = "sqlite.db"

# Connect to database
conn = sqlite3.connect(db_file)

# Create cursors
cursor = conn.cursor()

# Fetch the list of tables from database
cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
tables = cursor.fetchall()

# Fetch all rows from the table "result" with added information from "tenant" table such as contact_person, limitations, etc.
cursor.execute(f"""
               SELECT 
                    res.uuid AS result_uuid,
                    res.parameters AS result_paramters,
                    res.posting_tenant_uuid,
                    res.status,
                    ten.name AS tenant_name,
                    ten.contact_person AS tenant_contact,
                    ten.limitations AS tenant_limitations,
                    ten.is_active
               FROM result AS res 
               INNER JOIN tenant AS ten
               ON ten.uuid == res.posting_tenant_uuid;""")
rows = cursor.fetchall()

for i, row in enumerate(rows):
    print(f"row number {i}\n{row}")

# Close the connections
conn.close()
````

For further information, please contact Simon Krarup Steensen (skste@dtu.dk) at Technical University of Denmark (DTU).

# Ontologized results - simplified_results_singletask_ontologized.jsonld

This file offers a simplified view of the results enriched with ontologized concepts. 
Integrating ontologized concepts into data, as seen in simplified_results_singletask_ontologized.jsonld, enhances semantic understanding and interoperability across systems.
By structuring data with ontologies, the data representation facilitates more precise and standardized data interpretation and exchange. Furthermore the graph structure allow
for direct querying (such as SPARQL).

The implementation uses the BattINFO vocabulary which has been developed for creating semantic descriptions related to battery research. The file contains the results that are posted relating to requests from the optimizer i.e. conductivity. Input, measurements and results are then enriched with concepts from BattINFO. Additionally concepts from the schema.org vocabulary is used to enfer the overall structure. All data is encompassed in the type shcema:DataCatalog, with each individual posting defined as a schema:Dataset.

The file can be loaded into a graph, for either visualization purposes but more importantly allows for sematic queries. Using e.g. SPARQL one could write a query to obtain all nodes related to a specific electrolyte formulation, person thorugh their DOI, Organizations using ROR-IDs.

- For visulization of the JSON-LD use the https://json-ld.org/playground/. The json-ld playground is not great at visualizing huge graphs, so in the list 

"Datasets": [
    {...},
    {...},
    ...
]

remove all but one of the dictionaries, which makes the visualization much easier to understand.
- The file can be loaded into a graph in Python using the package pdflib. The package also allow querying using the SPARQL query language. https://rdflib.readthedocs.io/en/stable/


# Warning

Please ignore [version 18](https://archive.big-map.eu/records/b3dr2-1nm08) of this archive's entry, as the data files were generated from another empty database.   

